@@ -27,7 +27,7 @@ def parse_code(value):
         return lines
     line = lines[1]
     n = len(line) - len(line.lstrip())
-    return [l[:n].lstrip() + l[n:] for l in lines]
+    return [s[:n].lstrip() + s[n:] for s in lines]
 
 
 # region: block_builders
@@ -48,7 +48,7 @@ class CodeExtension(object):
     def __init__(self, token_start='@'):
 
         self.lexer_rules = {
-            300: (re.compile(r'%s(?=\()' % token_start), code_token),
+            300: (re.compile(r'\s*%s(?=\()' % token_start), code_token),
         }
 
     parser_rules = {
