@@ -16,6 +16,7 @@ compound_tokens = ['for ', 'if ', 'def ', 'extends'] + continue_tokens
 reserved_tokens = ['require', '#', 'include', 'import ', 'from ']
 all_tokens = end_tokens + compound_tokens + reserved_tokens
 out_tokens = ['markup', 'var', 'include']
+extends_tokens = ['def ', 'require', 'import ', 'from ']
 known_var_filters = {
     's': PY3 and 'str' or 'unicode'
 }
@@ -33,8 +34,8 @@ def stmt_token(m):
     return m.end(), str(m.group(2)), str(m.group(1)).replace('\\\n', '')
 
 
-RE_VAR = re.compile('(\.\w+)+')
-RE_VAR_FILTER = re.compile('(?<!!)!\w+(!\w+)*')
+RE_VAR = re.compile(r'(\.\w+)+')
+RE_VAR_FILTER = re.compile(r'(?<!!)!\w+(!\w+)*')
 
 
 def var_token(m):
@@ -51,7 +52,7 @@ def var_token(m):
         if not m:
             break
         pos = m.end()
-    value = str(source[start:end])
+    value = source[start:end]
     m = RE_VAR_FILTER.match(source, end)
     if m:
         end = m.end()
@@ -62,7 +63,7 @@ def var_token(m):
 def rvalue_token(m):
     """ Produces variable token as r-value expression.
     """
-    return m.end(), 'var', str(m.group(1).strip())
+    return m.end(), 'var', m.group(1).strip()
 
 
 # region: parser config
@@ -114,17 +115,19 @@ def parse_var(value):
 
 def build_extends(builder, lineno, token, nodes):
     assert token == 'render'
-    if len(nodes) != 1:
+    n = len(nodes)
+    if not n:
         return False
-    lineno, token, value = nodes[0]
+    lineno, token, value = nodes[-1]
     if token != 'extends':
         return False
-    extends, nodes = value
-    for lineno, token, value in nodes:
-        if token in ('def ', 'require', 'import ', 'from '):
+    extends, nested = value
+    if n > 1:
+        nested = nodes[:-1] + nested
+    for lineno, token, value in nested:
+        if token in extends_tokens:
             builder.build_token(lineno, token, value)
-    lineno = builder.lineno
-    builder.add(lineno + 1, 'return _r(' + extends +
+    builder.add(builder.lineno + 1, 'return _r(' + extends +
                 ', ctx, local_defs, super_defs)')
     return True
 
@@ -147,8 +150,8 @@ def build_import(builder, lineno, token, value):
 def build_from(builder, lineno, token, value):
     assert token == 'from '
     name, var, alias = value
-    builder.add(lineno, alias + ' = _i(' + name
-                + ').local_defs[\'' + var + '\']')
+    builder.add(lineno, alias + ' = _i(' + name +
+                ').local_defs[\'' + var + '\']')
     return True
 
 
@@ -269,18 +272,31 @@ def build_def(builder, lineno, token, value):
 
 def build_out(builder, lineno, token, nodes):
     assert token == 'out'
-    for lineno, token, value in nodes:
-        if token == 'include':
-            builder.add(lineno, 'w(_r(' + value +
+    builder.build_block(nodes)
+    return True
+
+
+def build_include(builder, lineno, token, value):
+    assert token == 'include'
+    builder.add(lineno, 'w(_r(' + value +
                         ', ctx, local_defs, super_defs))')
-        elif token == 'var':
-            var, var_filters = value
-            if var_filters:
-                for f in var_filters:
-                    var = known_var_filters.get(f, f) + '(' + var + ')'
-            builder.add(lineno, 'w(' + var + ')')
-        elif value:
-            builder.add(lineno, 'w(' + value + ')')
+    return True
+
+
+def build_var(builder, lineno, token, value):
+    assert token == 'var'
+    var, var_filters = value
+    if var_filters:
+        for f in var_filters:
+            var = known_var_filters.get(f, f) + '(' + var + ')'
+    builder.add(lineno, 'w(' + var + ')')
+    return True
+
+
+def build_markup(builder, lineno, token, value):
+    assert token == 'markup'
+    if value:
+        builder.add(lineno, 'w(' + value + ')')
     return True
 
 
@@ -295,12 +311,14 @@ def build_compound(builder, lineno, token, value):
 
 
 def build_require(builder, lineno, token, variables):
+    assert token == 'require'
     builder.add(lineno, '; '.join([
                 name + " = ctx['" + name + "']" for name in variables]))
     return True
 
 
 def build_comment(builder, lineno, token, comment):
+    assert token == '#'
     builder.add(lineno, comment)
     return True
 
@@ -394,6 +412,9 @@ class CoreExtension(object):
         ('from ', build_from),
         ('require', build_require),
         ('out', build_out),
+        ('markup', build_markup),
+        ('var', build_var),
+        ('include', build_include),
         ('def ', build_def_syntax_check),
         ('def ', build_def_empty),
         ('def ', build_def_single_markup),
